/*
    Workaround for https://github.com/Leaflet/Leaflet/issues/3575
*/
.leaflet-tile-container img {
    box-shadow: 0 0 1px rgba(0, 0, 0, 0.05);
}

/*
    Fix leaflet button styles
*/
.leaflet-bar button {
    margin: 0;
    padding: 0;
}

/*
    Leaflet marker styles
*/
.vector-marker {
    width: 30px;
    height: 39px;
    position: absolute;
    left: 0;
    top: 0;
    display: block;
    text-align: center;
    -webkit-filter: drop-shadow(1px 0px 2px #999);
    filter: drop-shadow(1px 0px 2px #999);
}

.vector-marker i {
    position: relative;
    top: 7px;
    width: 100%;
    color: #333;
    margin: 0;
    padding: 0;
    text-align: center;
    display: inline-block;
    font-size: 14px;
    z-index: 1000;
}

/*
    Make sure the attribution is always rendered as one line
*/

.leaflet-control-attribution ul {
    list-style: none;
    margin: 0;
}

.leaflet-control-attribution ul li {
    float: left;
    font-size: 10px !important;
    font-weight: normal;
    margin-bottom: 3px;
    padding: 0;
    position: relative;
    top: 1px;
}

.leaflet-control-attribution ul li + li {
    margin-left: 5px;
}

.leaflet-control-attribution ul li a {
    font-weight: normal;
    color: #333;
    line-height: 10px;
}

.leaflet-control-attribution ul li a:hover {
    color: #2D5594;
}

/*
    Map button context menu
*/

.easy-button-container {
    background-color: white;
}

button.open-active {
    float: right;
}

.leaflet-container .leaflet-control-container .map-context-menu {
    float: left;
    list-style: none;
    margin: 0;
    padding: 0;
    text-align: right;
}

.leaflet-container .leaflet-control-container .map-context-menu li a,
.leaflet-container .leaflet-control-container .map-context-menu li a:hover {
    font-size: 16px !important;
    font-weight: normal !important;
    padding: 0 5px;
    text-align: left;
    text-overflow: normal !important;
    width: auto !important;
}

/*
    Disable mouse pointer on non-clickable markers (which is currently all)
*/

.marker-undefined {
    cursor: -webkit-grab;
    cursor: -moz-grab;
    cursor: grab;
}

.marker-thumbnail {
    cursor: auto;
}